<?php

namespace Trello\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

use App\Trello\Models\CustomFields\{Action,SalesStage};

final class Card extends Model
{
	protected $keyType = 'string';

	/**
	 * Get this card's custom field value.
	 *
	 * @param CustomField $o
	 * @return object|null
	 */
	public function getCustomFieldValue(CustomField $o): ?object
	{
		return collect($this->customFieldItems)->filter(function($item) use ($o) { return $item->idCustomField == $o->id; })->pop();
	}

	/**
	 * Determine if this card should be ignored.
	 *
	 * @param Board $b
	 * @return bool
	 * @throws \Exception
	 */
	public function shouldIgnore(Board $b): bool
	{
		// Check Stages
		$cf = $b->customField('Stage');
		$y = '';

		if (($x=$this->getCustomFieldValue($cf)) && (! in_array($y=$cf->value($x),SalesStage::options))) {
			Log::info(sprintf('Card [%s] Stage value is not one of ours [%s]',$this->id,$y));

			return TRUE;
		}
		Log::debug(sprintf('Card [%s] Stage value is [%s]',$this->id,$y ?: 'Not Set'));

		// Check Actions
		$cf = $b->customField('Action');
		$y = '';

		if (($x=$this->getCustomFieldValue($cf)) && (! in_array($y=$cf->value($x),Action::options)) || ($y === 'Ignore')) {
			Log::info(sprintf('Card [%s] Action value is not one of ours [%s]',$this->id,$y));

			return TRUE;
		}
		Log::debug(sprintf('Card [%s] Action value is [%s]',$this->id,$y ?: 'Not Set'));

		return FALSE;
	}
}