<?php

namespace Trello\Models;

use Trello\Models\CustomFields\{Base,Checkbox,Date,ListList,Number,Text};

abstract class CustomField extends Base
{
	protected $keyType = 'string';
	private const LOGKEY = 'TCF';

	/**
	 * Resolve the data into a CustomField Object
	 *
	 * @param object $data
	 * @return mixed|Checkbox|Date|ListList|Number|Text
	 * @throws \Exception
	 */
	public static function factory(object $data) {
		// See if the field name is one we have configured
		if (collect(config('trello.customfields'))->has($x=object_get($data,'name'))) {
			$xx = config('trello.customfields.'.$x);
			$o = new $xx($data);

			if ($o->trello_type !== ($y=object_get($data,'type')))
				throw new \Exception(sprintf('%s:! ERROR - Custom field [%s] (%s) is not the right type [%s]',self::LOGKEY,$x,$o->trello_type,$y));

			return $o;
		}

		switch ($x=object_get($data,'type')) {
			case 'checkbox':
				return new Checkbox($data);

			case 'date':
				return new Date($data);

			case 'list':
				return new ListList($data);

			case 'number':
				return new Number($data);

			case 'text':
				return new Text($data);

			default:
				dump($data);
				throw new \Exception('Unknown data type: '.$x);
		}
	}

	/**
	 * Set a customfield's value
	 *
	 * @param string $value
	 * @return array
	 */
	abstract public function set(string $value): array;

	/**
	 * Get a customfield's value
	 *
	 * @param object $key
	 * @return string|null
	 */
	abstract public function value(object $key): ?string;
}