<?php

namespace Trello\Models\CustomFields;

use Illuminate\Support\Collection;
use Trello\Models\CustomField;
use Trello\Models\CustomFields\Options\Option;

class ListList extends CustomField
{
	public Collection $options;

	public function __construct(object $attributes)
	{
		parent::__construct($attributes);

		$this->options = collect();

		foreach (object_get($attributes,'options',[]) as $option)
			$this->addOption((new Option)->forceFill((array)$option)->syncOriginal());

		unset($this->attributes->options);
	}

	/* ABSTRACT METHODS */

	public function set(string $value): array
	{
		return ['idValue'=>$this->id($value)];
	}

	/**
	 * Get the value, given a customfield key
	 *
	 * @param object $key
	 * @return string|null
	 */
	public function value(object $key): ?string
	{
		return ($x=$this->options->filter(function($item) use ($key) { return $item->id == $key->idValue; })->pop()) ? $x->value->text : NULL;
	}

	/* METHODS */

	public function addOption(Option $option): void
	{
		$this->options->push($option);
	}

	/**
	 * Make sure our list has all the option values
	 *
	 * @return bool
	 */
	public function hasAllOptions(): bool
	{
		return $this->missingOptions()->count() == 0;
	}

	private function id(string $name): ?string
	{
		return ($x=$this->options->filter(function($item) use ($name) { return $item->name == $name; })->pop()) ? $x->id : NULL;
	}

	public function missingOptions(): Collection
	{
		return collect(static::options)->diff($this->options->pluck('name'));
	}

}