<?php

namespace Trello\Response;

use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Log;

/**
 * This parent class handles responses received from Trello
 *
 * @note: This class is used for events not specifically created.
 */
class Base implements \JsonSerializable
{
	protected const LOGKEY = 'RB-';

	/**
	 * Default Constructor Setup
	 *
	 * @param object $response
	 */
	public function __construct(array $response)
	{
		$this->_data = $response;

		// This is only for child classes
		if (get_class($this) == Base::class) {
			Log::debug(sprintf('%s:Trello RESPONSE Initialised [%s]',static::LOGKEY,get_class($this)),['m'=>__METHOD__]);

			if (App::environment() == 'dev')
				file_put_contents('/tmp/response',print_r($this,TRUE),FILE_APPEND);
		}
	}

	/**
	 * Enable getting values for keys in the response
	 *
	 * @note: This method is limited to certain values to ensure integrity reasons
	 * @note: Classes should return:
	 *    + channel_id,
	 *    + team_id,
	 *    + ts,
	 *    + user_id
	 */
	public function __get($key)
	{
		switch ($key) {
			case 'id': 
				return object_get($this->_data,$key);
		}
	}

	/**
	 * When we json_encode this object, this is the data that will be returned
	 */
	public function jsonSerialize()
	{
		return $this->_data ? $this->_data : new \stdClass;
	}
}